VERSION 5.00
Begin VB.UserControl cSysTray 
   Appearance      =   0  'Flat
   BorderStyle     =   1  'Fixed Single
   CanGetFocus     =   0   'False
   ClientHeight    =   480
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   480
   ClipControls    =   0   'False
   EditAtDesignTime=   -1  'True
   InvisibleAtRuntime=   -1  'True
   MouseIcon       =   "SysTray.ctx":0000
   Picture         =   "SysTray.ctx":030A
   ScaleHeight     =   32
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   32
   ToolboxBitmap   =   "SysTray.ctx":0BD4
End
Attribute VB_Name = "cSysTray"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Constants
Private Const MAX_SIZE = 500
Private Const defInTray = False
Private Const defTrayTip = "" & vbNullChar

' Control Variables (used to store property values)
Private S_InTray As Boolean
Private S_TrayID As Long
Private S_ToolTipText As String
Private S_hWnd As Long
Private S_TrayIcon As StdPicture
Private S_AddedToTray As Boolean


'================================================================================
'                               CONTROL EVENTS
'================================================================================


Public Event MouseMove(ID As Long)
Attribute MouseMove.VB_Description = "Mouse Move - Fires when the user moves their cursor over the TrayIcon in the System Tray."
Public Event MouseDown(Button As Integer, ID As Long)
Attribute MouseDown.VB_Description = "Mouse Down Event - Fires when the user presses down on the left, middle, or right mouse button while over the Tray Icon in the System Tray."
Public Event MouseUp(Button As Integer, ID As Long)
Attribute MouseUp.VB_Description = "Mouse Up Event - Fires when the user releases a pressed mouse button (left, middle, or right) while over the Tray Icon in the System Tray."
Public Event MouseDblClick(Button As Integer, ID As Long)
Attribute MouseDblClick.VB_Description = "Mouse Double Click Event - Fires when the user double clicks on the TrayIcon in the System Tray."


'================================================================================
'                            CONTROL INITIALIZATION
'================================================================================


Private Sub UserControl_Initialize()
On Error Resume Next
  
  S_InTray = defInTray  ' Set default InTray property
  S_AddedToTray = False ' Set default state
  S_TrayID = 0          ' Set default TrayID
  S_hWnd = hwnd         ' Set and keep HWND of user control
  
End Sub

Private Sub UserControl_Terminate()
On Error Resume Next
  
  ' Clear up
  If InTray Then
    InTray = False
  End If
  
End Sub

Private Sub UserControl_InitProperties()
On Error Resume Next
  
  InTray = defInTray     ' Init InTray Property
  TrayTip = defTrayTip   ' Init TrayTip Property
  Set TrayIcon = Picture ' Init TrayIcon property
  
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
On Error Resume Next
  
  ' Read in the properties that have been saved into the PropertyBag...
  With PropBag
    InTray = .ReadProperty("InTray", defInTray)       ' Get InTray
    Set TrayIcon = .ReadProperty("TrayIcon", Picture) ' Get TrayIcon
    TrayTip = .ReadProperty("TrayTip", defTrayTip)    ' Get TrayTip
  End With
  
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
On Error Resume Next
  
  With PropBag
    .WriteProperty "InTray", S_InTray       ' Save InTray to propertybag
    .WriteProperty "TrayIcon", S_TrayIcon   ' Save TrayIcon to propertybag
    .WriteProperty "TrayTip", S_ToolTipText ' Save TrayTip to propertybag
  End With
  
End Sub

Private Sub UserControl_Resize()
On Error Resume Next
  
  ' Prevent Control from being resized
  Height = MAX_SIZE + (Screen.TwipsPerPixelY * 2)
  Width = MAX_SIZE
  
End Sub


'================================================================================
'                              CONTROL PROPERTIES
'================================================================================


' Return the InTray value
Public Property Get InTray() As Boolean
Attribute InTray.VB_Description = "Returns/sets a value that determines if the specified TrayIcon will be visible in the System Tray."
On Error Resume Next
  
  InTray = S_InTray
  
End Property

' Sets the InTray value
Public Property Let InTray(NewValue As Boolean)
On Error Resume Next
  
  Dim ClassAddr As Long ' Address pointer to Control Instance
  
  If (NewValue <> S_InTray) Then ' Modify ONLY if state is changing!
    If NewValue Then             ' If adding Icon to system tray...
      If Ambient.UserMode Then   ' If in RunMode and not in IDE...
        
        ' SubClass Controls window proc.
        PrevWndProc = SetWindowLong(S_hWnd, GWL_WNDPROC, AddressOf SubWndProc)
        
        ' Get address to user control object
       'CopyMemory ClassAddr, UserControl, 4&
        
        ' Save address to the USERDATA of the control's window struct.  This will
        ' be used to get an object refenence to the control from an HWND in the callback.
        SetWindowLong S_hWnd, GWL_USERDATA, ObjPtr(Me) 'ClassAddr
        
        ' Add TrayIcon to System Tray
        AddIcon S_hWnd, S_TrayID, TrayTip, TrayIcon
        
        ' Save state of control used in teardown procedure
        S_AddedToTray = True
      End If
    Else                            ' If removing Icon from system tray
      If S_AddedToTray Then         ' If Added to system tray then remove...
        DeleteIcon S_hWnd, S_TrayID ' Remove icon from system tray
        
        ' Un SubClass controls window proc.
        SetWindowLong S_hWnd, GWL_WNDPROC, PrevWndProc
        
        ' Maintain the state for teardown purposes
        S_AddedToTray = False
      End If
    End If
    
    ' Update global variable
    S_InTray = NewValue
    
    ' Notify control that property has changed
    PropertyChanged "InTray"
  End If
  
End Property

' Returns the picture stored in the TrayIcon property
Public Property Get TrayIcon() As StdPicture
Attribute TrayIcon.VB_Description = "Returns/sets the icon to be displayed in the System Tray."
On Error Resume Next
  
  Set TrayIcon = S_TrayIcon
  
End Property

' Sets the picture stored in the TrayIcon property
Public Property Set TrayIcon(NewValue As StdPicture)
On Error Resume Next
  
  Dim Tray As NOTIFYICONDATA
  
  If Not (NewValue Is Nothing) Then         ' If icon is valid...
    If (NewValue.Type = vbPicTypeIcon) Then ' Use ONLY if it is an icon
      If S_AddedToTray Then                 ' Modify tray only if it is in use.
        Tray.uID = S_TrayID                 ' Unique ID for each HWND and callback message.
        Tray.hwnd = S_hWnd                  ' HWND receiving messages.
        Tray.hIcon = NewValue.Handle        ' Tray icon.
        Tray.uFlags = NIF_ICON              ' Set flags for valid data items
        Tray.cbSize = Len(Tray)             ' Size of struct.
        
        Shell_NotifyIcon NIM_MODIFY, Tray   ' Send data to Sys Tray.
      End If
      
      Set S_TrayIcon = NewValue             ' Save Icon to global
      Set Picture = NewValue                ' Show user change in control as well(gratuitous)
      PropertyChanged "TrayIcon"            ' Notify control that property has changed.
    End If
  End If
  
End Property

' Returns the current ToolTipText
Public Property Get TrayTip() As String
On Error Resume Next
  
  TrayTip = S_ToolTipText
  
End Property

' Sets the current ToolTipText
Public Property Let TrayTip(NewValue As String)
Attribute TrayTip.VB_Description = "Returns/sets the ToolTipText that will show when you hold the cursor over the TrayIcon in the System Tray."
Attribute TrayTip.VB_ProcData.VB_Invoke_PropertyPut = ";Misc"
Attribute TrayTip.VB_UserMemId = -517
On Error Resume Next
  
  Dim Tray As NOTIFYICONDATA
  
  If S_AddedToTray Then                ' If TrayIcon is in taskbar
    Tray.uID = S_TrayID                ' Unique ID for each HWND and callback message.
    Tray.hwnd = S_hWnd                 ' HWND receiving messages.
    Tray.szTip = NewValue & vbNullChar ' Tray tool tip
    Tray.uFlags = NIF_TIP              ' Set flags for valid data items
    Tray.cbSize = Len(Tray)            ' Size of struct.
    
    Shell_NotifyIcon NIM_MODIFY, Tray  ' Send data to Sys Tray.
  End If
  
  S_ToolTipText = NewValue             ' Save Tip
  PropertyChanged "TrayTip"            ' Notify control that property has changed
  
End Property


'================================================================================
'                              CONTROL METHODS
'================================================================================


Public Sub About()
Attribute About.VB_Description = "Shows SysTray's about screen."
Attribute About.VB_UserMemId = -552
  
  frmAbout.Show vbModal
  
End Sub

Private Sub AddIcon(hwnd As Long, ID As Long, Tip As String, Icon As StdPicture)
On Error Resume Next
  
  Dim Tray As NOTIFYICONDATA
  Dim tFlags As Long
  
  Tray.uID = ID                            ' Unique ID for each HWND and callback message.
  Tray.hwnd = hwnd                         ' HWND receiving messages.
  
  If Not (Icon Is Nothing) Then            ' Validate Icon picture
    Tray.hIcon = Icon.Handle               ' Tray icon.
    Tray.uFlags = Tray.uFlags Or NIF_ICON  ' Set ICON flag to validate data item
    Set S_TrayIcon = Icon                  ' Save icon
  End If
  
  If (Tip <> "") Then                      ' Validate Tip text
    Tray.szTip = Tip & vbNullChar          ' Tray tool tip
    Tray.uFlags = Tray.uFlags Or NIF_TIP   ' Set TIP flag to validate data item
    S_ToolTipText = Tip                    ' Save tool tip
  End If
  
  Tray.uCallbackMessage = TRAY_CALLBACK    ' Set user defigned message
  Tray.uFlags = Tray.uFlags Or NIF_MESSAGE ' Set flags for valid data item
  Tray.cbSize = Len(Tray)                  ' Size of struct.
  
  Shell_NotifyIcon NIM_ADD, Tray           ' Send data to Sys Tray.
  
End Sub

Private Sub DeleteIcon(hwnd As Long, ID As Long)
On Error Resume Next
  
  Dim Tray As NOTIFYICONDATA
  
  Tray.uID = ID                     ' Unique ID for each HWND and callback message.
  Tray.hwnd = hwnd                  ' HWND receiving messages.
  Tray.uFlags = 0&                  ' Set flags for valid data items
  Tray.cbSize = Len(Tray)           ' Size of struct.
  
  Shell_NotifyIcon NIM_DELETE, Tray ' Send delete message.
  
End Sub

Friend Sub SendEvent(MouseEvent As Long, ID As Long)
On Error Resume Next
  
  ' Send mouse event depending on which event was recieved
  Select Case MouseEvent
    Case WM_MOUSEMOVE
      RaiseEvent MouseMove(ID)
    Case WM_LBUTTONDOWN
      RaiseEvent MouseDown(vbLeftButton, ID)
    Case WM_LBUTTONUP
      RaiseEvent MouseUp(vbLeftButton, ID)
    Case WM_LBUTTONDBLCLK
      RaiseEvent MouseDblClick(vbLeftButton, ID)
    Case WM_MBUTTONDOWN
      RaiseEvent MouseDblClick(vbMiddleButton, ID)
    Case WM_MBUTTONUP
      RaiseEvent MouseDblClick(vbMiddleButton, ID)
    Case WM_MBUTTONDBLCLK
      RaiseEvent MouseDblClick(vbMiddleButton, ID)
    Case WM_RBUTTONDOWN
      RaiseEvent MouseDown(vbRightButton, ID)
    Case WM_RBUTTONUP
      RaiseEvent MouseUp(vbRightButton, ID)
    Case WM_RBUTTONDBLCLK
      RaiseEvent MouseDblClick(vbRightButton, ID)
  End Select
  
End Sub
